#include "PAConfig.h"


PAConfig::PAConfig() {

}

void PAConfig::load(File _config) {
  if (!_config)
    return;
  const size_t capacity = _config.size();
  if (capacity > 1024)
    return;

  std::unique_ptr<char[]> buf(new char[capacity]);
  _config.readBytes(buf.get(), capacity);
  DynamicJsonBuffer jsonBuffer(capacity);
  JsonObject& root = jsonBuffer.parseObject(buf.get());

  if (!root.success())
    return;

  this->gmail.username          = root["gmail"]["username"].as<String>(); // "username-string"
  this->gmail.password          = root["gmail"]["password"].as<String>(); // "password-string"

  JsonObject& weather           = root["weather"];
  this->weather.woeid           = weather["woeid"].as<String>(); // woeid
  this->weather.api             = weather["api"].as<String>(); // "api-string"
  this->weather.appId           = weather["appId"].as<String>(); // "app-id-string"
  this->weather.consumerKey     = weather["consumerKey"].as<String>(); // "consumer-key-string"
  this->weather.consumerSecret  = weather["consumerSecret"].as<String>(); // "consumer-secret-string"

  this->time.server             = root["time"]["server"].as<String>(); // "time-server-string"
  this->time.timezone           = root["time"]["timezone"]; // 3.5

  this->wifi.ssid               = root["wifi"]["ssid"].as<String>(); // "wifi-ssid-string"
  this->wifi.password           = root["wifi"]["password"].as<String>(); // "wifi-password-string"

  _config.close();
}

void PAConfig::save(File _config) {
  if (!_config)
    return;

  const size_t capacity = 3*JSON_OBJECT_SIZE(2) + JSON_OBJECT_SIZE(4) + JSON_OBJECT_SIZE(5);
  DynamicJsonBuffer jsonBuffer(capacity);

  JsonObject& root = jsonBuffer.createObject();

  JsonObject& gmail = root.createNestedObject("gmail");
  gmail["username"] = this->gmail.username;
  gmail["password"] = this->gmail.password;

  JsonObject& weather = root.createNestedObject("weather");
  weather["woeid"]          = this->weather.woeid;
  weather["api"]            = this->weather.api;
  weather["appId"]          = this->weather.appId;
  weather["consumerKey"]    = this->weather.consumerKey;
  weather["consumerSecret"] = this->weather.consumerSecret;

  JsonObject& time = root.createNestedObject("time");
  time["server"]    = this->time.server;
  time["timezone"]  = this->time.timezone;

  JsonObject& wifi = root.createNestedObject("wifi");
  wifi["ssid"]      = this->wifi.ssid;
  wifi["password"]  = this->wifi.password;

  root.printTo(_config);

  _config.close();
}
