#pragma once

#include "Arduino.h"
#include "ArduinoJson.h"
#include "FS.h"
#include "DFRobotDFPlayerMini.h"

#define GMAIL_UNREADMESSAGES          0
#define BIRTHDEATH_WORLDBIRTH         1
#define BIRTHDEATH_WORLDDEATH         2
#define WEATHER_CURRENTTEMPERATURE    3
#define WEATHER_TODAYCONDITION        4
#define WEATHER_TODAYMINMAX           5
#define WEATHER_TOMORROWCONDITION     6
#define WEATHER_TOMORROWMINMAX        7
#define WEATHER_PRECIPITATIONFORECAST 8
#define WEATHER_SUNRISESUNSET         9
#define TIME_CURRENTTIME              10
#define TIME_CURRENTDATE              11

#define calcBirth(t) (t) * 4
#define calcDeath(t) (t) * 2
#define LEAP_YEAR(Y)     ( ((1970+(Y))>0) && !((1970+(Y))%4) && ( ((1970+(Y))%100) || !((1970+(Y))%400) ) )

class PersonalAssistant {

    public:

        PersonalAssistant(DFRobotDFPlayerMini& player);
        void numberToWord(int32_t number, uint8_t words[]);
        void loadServices(File _services);
        void saveServices(File _services);
        void setModule(uint8_t module, bool enable);

        // Speak Queue
        void    handleSpeak();
        uint8_t getQueueTurn();
        void    speakQAdd(uint8_t item, uint16_t duration);
        uint8_t speakQLoad();
        void    speakQClear();
        bool    speakQisEmpty();

        static uint8_t calcNumSpeakDuration(uint8_t number);
        static void getDate(uint32_t time, uint16_t date[]);

        static const uint8_t N2W_HUNDRED  = 0XB3;
        static const uint8_t N2W_THOUSAND = 0XB6;
        static const uint8_t N2W_AND      = 0XC0;
        static const uint8_t N2W_MINUS    = 0XC1;
        static const uint8_t N2W_NIL      = 0XFF;

    private:

      void number2word(uint8_t number, uint8_t wheelbarrows[]);

      DFRobotDFPlayerMini* dfPlayer;

      // Speak Queue
      uint8_t   speakQueue[32];
      uint16_t  durationQueue[32];
      uint8_t   speakQStart = 0;
      uint8_t   speakQEnd   = 0;
      uint32_t  speakTimestamp = 0;

      uint8_t   _turn = 0;

      uint8_t _ones[10]             = { 0XC2, 0X01, 0X02, 0X03, 0X04, 0X05, 0X06, 0X07, 0X08, 0X09 }; // @N2W "zero", "one", ..., "nine"
      uint8_t _tensExceptional[11]  = { 0XFF, 0X10, 0X11, 0X12, 0X13, 0X14, 0X15, 0X16, 0X17, 0X18, 0X19 }; // @N2W "", "ten", "eleven", "twelve", ..., "nineteen"
      uint8_t _tensMultiple[10]     = { 0XFF, 0XFF, 0XA2, 0XA3, 0XA4, 0XA5, 0XA6, 0XA7, 0xA8, 0XA9 }; // @N2W "", "", "twenty", "thirty", "forty", ..., "ninety"
      // uint8_t _tensPower[2] = { 0XB3, 0XB6 }; // @N2W "hundred", "thousand"
      // uint8_t _special[1] = { 0XC0 }; // @N2W "and"

      bool _moduleQueue[12];

};
